<?php

namespace Drupal\single_page_protection\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;

class SinglePageProtectionController extends ControllerBase {

    /**
     * This method performs access control for protected pages.
     * It checks if the current page is protected and, if so, prompts the user for a password.
     */
    public function handlePageAccess(Request $request) {
        $current_path = $request->getPathInfo();
        $protected_page = $this->getProtectedPageInfo($current_path);

        if ($protected_page) {
            // Display a form requesting the password from the user.
            return $this->formBuilder()->getForm('Drupal\single_page_protection\Form\ProtectedPagePasswordForm', $protected_page);
        }

        // If the page is not protected, allow normal access.
        // You can use Drupal's content management system to load the normal page content.
        return $this->redirect('<front>'); // For demonstration, redirecting to the homepage.
    }

    /**
     * Loads the information of a protected page from the database.
     * This includes the URL of the page and its associated password.
     */
    private function getProtectedPageInfo($path) {
        $result = \Drupal::database()->select('single_page_protection', 'n')
            ->fields('n', ['url', 'password'])
            ->condition('url', $path)
            ->execute()
            ->fetchAssoc();

        return $result ? $result : NULL;
    }
}
