<?php

namespace Drupal\single_page_protection\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;

class ProtectedPagePasswordForm extends FormBase {

    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'single_page_protection_password_form';
    }

    /**
     * Builds the form for password entry for a protected page.
     */
    public function buildForm(array $form, FormStateInterface $form_state, $protected_page = NULL) {
        $form['password'] = [
            '#type' => 'password',
            '#title' => $this->t('Password'),
            '#required' => TRUE,
        ];

        $form['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Submit'),
        ];

        // Store the URL and password of the protected page.
        $form_state->set('protected_page', $protected_page);

        return $form;
    }

    /**
     * Handles the form submission.
     * Verifies the entered password against the stored value and redirects to the protected page if correct.
     */
    public function submitForm(array &$form, FormStateInterface $form_state) {
        $current_request = \Drupal::request();
        $protected_page = $current_request->query->get('path');
        $input_password = $form_state->getValue('password');

        $result = \Drupal::database()->select('single_page_protection', 'n')
            ->fields('n', ['url', 'password'])
            ->condition('url', $protected_page)
            ->execute()
            ->fetchAssoc();

        if ($result && ($input_password === $result['password'])) {
            // Correct password, redirect to the protected page.
            $_SESSION['single_page_protection'][$protected_page] = TRUE;
            $form_state->setRedirectUrl(\Drupal\Core\Url::fromUserInput($protected_page));
        } else {
            // Incorrect password, display an error message.
            \Drupal::messenger()->addError($this->t('Incorrect password.'));
        }
    }
}
